library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library std;
use std.env.all;
use std.textio.all;

entity pulseGeneratorTime_tb is
end entity;

architecture testbench of pulseGeneratorTime_tb is

    constant c_ClkFrequency: positive := 200e6; -- in Hz
    constant c_ClockPeriod: time := (1.0 sec)/c_ClkFrequency;

    constant c_PulseMinWidthInTime: time := 1 ns;

    signal Clk_k, Reset_r, SignalIn, PulseOut: std_logic := '0';
    signal LastPulseStart, LastPulseWidth: time := 0 ns;

    procedure f_tick(Ticks: in natural) is begin
        wait for Ticks * c_ClockPeriod;
    end procedure;

begin

    i_Dut: entity work.pulseGeneratorTime(syn)
        generic map (
            g_ClkFrequency => c_ClkFrequency,
            g_PulseMinWidthInTime => c_PulseMinWidthInTime
        )
        port map (
            Clk_ik => Clk_k,
            Reset_ir => Reset_r,
            Signal_i => SignalIn,
            Pulse_o => PulseOut
        );

    pClk: process is begin
        Clk_k <= '0';
        wait for c_ClockPeriod/2;
        Clk_k <= '1';
        wait for c_ClockPeriod/2;
    end process;

    pPulseWidth: process (PulseOut) is begin
        if rising_edge(PulseOut) then
            LastPulseStart <= now;
        else
            LastPulseWidth <= now - LastPulseStart;
            report "Last pulse width: " & time'image(now - LastPulseStart);
        end if;
    end process;

    pTest: process is begin
        -- init wait
        f_tick(2);
        -- try single output
        wait until Clk_k = '0';
        SignalIn <= '1';
        f_tick(1);
        SignalIn <= '0';
        wait on LastPulseWidth;
        assert LastPulseWidth >= c_PulseMinWidthInTime
            report "Pulse is too short! (1)"
            severity failure;
        -- try multiple inputs
        wait until Clk_k = '0';
        SignalIn <= '1';
        f_tick(1);
        SignalIn <= '0';
        wait for c_PulseMinWidthInTime/2;
        wait until Clk_k = '0';
        SignalIn <= '1';
        f_tick(1);
        SignalIn <= '0';
        wait on LastPulseWidth;
        assert LastPulseWidth >= c_PulseMinWidthInTime
            report "Pulse is too short! (1)"
            severity failure;
        assert false report "NONE. End of simulation." severity failure;
        wait;
    end process;

end architecture;
