from Spi import *
from CommonFunctions import *

class AD9512(Spi):

    _SpiConfig = {
            "Channel": 0,
            "CPol": 0,
            "CPha": 0,
            "LSB1st": 0,
            "Length": 24,
            "HalfPeriod": 10, # HalfPeriod=5 ~ 10 MHz
            "WaitTime": 10
        }

    def __init__(self, wbOverlay, SpiBaseAddress, SpiOffset = 0):
        super(AD9512, self).__init__(wbOverlay, self._SpiConfig, SpiBaseAddress, SpiOffset)

    def Write(self, Address, Data, SelfCommit = False):
        RnW = 0
        Length = 0 # only 1 byte transaction supported
        Address &= 2**13-1
        Data &= 2**8-1
        DataLow = (RnW<<23) + (Length<<21) + (Address<<8) + Data
        self._debug("AD9512 SPI Write: 0x%06x" % DataLow)
        self.SpiAccess(DataLow)
        if SelfCommit:
            self.Commit()

    def Read(self, Address):
        RnW = 1
        Length = 0 # only 1 byte transaction supported
        Address &= 2**13-1
        DataLow = (RnW<<23) + (Length<<21) + (Address<<8)
        self._debug("AD9512 SPI Read: 0x%06x" % DataLow)
        Data = self.SpiAccess(DataLow)
        self._debug("AD9512 SPI Read result: 0x%06x" % Data)
        Data &= 2**8-1
        return Data

    def Commit(self):
        self.Write(0x5a, 1)

    def Config(self, Config):
        for Register in Config:
            Address = Register[0]
            Data = Register[1]
            self.Write(Address, Data)
        self.Commit()

    def Check(self, Config):
        print "Checking AD9512 configuration..."
        errors = 0
        for Register in Config:
            Address = Register[0]
            Data = Register[1]
            DataRead = self.Read(Address)
            if Data <> DataRead:
                ErrorMsg("Data at address 0x%02x are different! Data read = 0x%02x, should be 0x%02x" % (Address, DataRead, Data))
                errors += 1
        if errors == 0:
            OkMsg("no error found")
        else:
            ErrorMsg("found %d errors!" % errors)
